unit dd086u1;

interface

//TO DO: Add test that the file is right format...
//     meSource.lines[7] should start "<!--FWDF:aa"

//Fix flawed logic in sFetchData, so that, say, -1.1,
//     nor -11.1 nor 100.0 cause problems

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls;

const ver='9jan11e';
//DD086 started 9 Jan 11 in conjunction with work to understand
//  Pachube.com, the "YouTube for data streams".
//  For "Getting started with Pachube, see http://sheepdogsoftware.co.uk/sc2wy.htm

type
  TDD086f1 = class(TForm)
    meSource: TMemo;
    Button1: TButton;
    laReloadInt: TLabel;
    eLoadFromP: TEdit;
    eLoadFromF: TEdit;
    Label2: TLabel;
    Label3: TLabel;
    Timer1: TTimer;
    buDoIt: TButton;
    Label1: TLabel;
    meOutput: TMemo;
    Label4: TLabel;
    eSaveToP: TEdit;
    eSaveToF: TEdit;
    Label5: TLabel;
    Label6: TLabel;
    laTmp: TLabel;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure buDoItClick(Sender: TObject);
  private
    { Private declarations }
    sTmp,sTmp2:string;
    rTmp:real48;//"real" will do. real48 probably better,
       //if you are using D.7 or above
    iErr:integer;
    procedure DoIt;
    function sFetchData:string;
    procedure FillOutputMemo(sTmp:string);
    procedure WriteMemoToFile;
    function sBuildPathPlusFile(sPath, sName:string):string;

  public
    { Public declarations }
  end;

var
  DD086f1: TDD086f1;

implementation

{$R *.dfm}

procedure TDD086f1.Button1Click(Sender: TObject);
begin
application.terminate;
end;

procedure TDD086f1.FormCreate(Sender: TObject);
begin

caption:='DD086- read data, create new file. SheepdogSoftware.co.uk '+ver;
meOutput.lines.clear;

//Put in ini file in due course....
eLoadFromP.text:='C:/wamp/www/';
eLoadFromF.text:='index.htm';
eSaveToP.text:='C:/wamp/www/';
eSaveToF.text:='dd086.xml';
Timer1.interval:=7*60000;// 7*60000: every 7 minutes... so with Pachube
   //fetching every 15, data never more than half that old.
laReloadInt.caption:='Reloads every '+inttostr(Timer1.interval)+' milli-seconds';
//End of stuff for inifile

end;//FormCreate

procedure TDD086f1.Timer1Timer(Sender: TObject);
begin
DoIt;
end;

procedure TDD086f1.DoIt;
begin
sTmp:=sBuildPathPlusFile(eLoadFromP.text,eLoadFromF.text);
if fileexists(sTmp) then begin
   meSource.Lines.LoadFromFile(sTmp);
   sTmp:=sFetchData;
   FillOutputMemo(sTmp);
   WriteMemoToFile;
   end // no ; here
  else begin
   meSource.Lines.clear;
   meSource.Lines.Add('-------------------------');
   meSource.Lines.Add('****!*!*!*!*!*!*!*!*!****');
   meSource.Lines.Add('');
   meSource.Lines.Add('Could not find a file called...');
   meSource.Lines.Add(sTmp);
   meSource.Lines.Add('... which was specified for input.');
   end;

end;

procedure TDD086f1.buDoItClick(Sender: TObject);
begin
DoIt;
end;

function TDD086f1.sFetchData:string;
(*The code here could consist of just...
    result:='TestOfFetchData';
  ... if you were just running tests.

  The complex code you see is suited to a particular
  version of the FarWatch system, with which DD086 is
  working on one of my "real world" setups, harvesting
  data from a file created by DS025, working with FarWatch.
  The harvested data is being passed to Pachube, as
  a second way for "the world" to see the data.

  meSource should be filled before this is called, and as this
  is currently written, assumptions are made about what will be
  there.

  Much of what follows is for changing readings from Fahrenheit
  to Celcius

  The line expected is in the form....

  73.4 F    46.4 F  --at--  09-Jan-2011, 13:31:11   1520284

  ... from which we want the "46.4 F", initially. It could be
  08.1 C, alternatively. We want 08.1 returned.

  Hmmm. Code at present assumes tture will always take exactly
     6 characters... what about -9 to +9 inclusive. Leadiong zeros?
     What about -12.0 or +102.0 F?

     BUG! At present probably only works if tture expressed in 6 chars,
        e.g. 42.3 F, -1.4 C, or, if leading zero given, 04.3 C, but
        NOT -04.2 C.
        Either look into application generating the data, or make DD086
        and DE046 more clever.
*)

begin
sTmp:=copy(meSource.lines[19],13,6);
sTmp2:=copy(sTmp,6,1);
sTmp:=copy(sTmp,1,4);

if sTmp2='F' then begin //convert F to C
  val(sTmp,rTmp,iErr);
  if iErr<>0 then begin
     rTmp:=99.9;//rogue value for "Conversion not possible"
     end//no ; here
    else begin
     rTmp:=(rTmp-32)/1.8;
     end;//else
  sTmp:=floattostrf(rTmp,ffFixed,7,1);
  end;// begin convert...

laTmp.caption:='xx'+sTmp+'xx';
result:=sTmp;
end;


procedure TDD086f1.FillOutputMemo(sTmp:string);
(*This procedure just fills the output memo with whatever
you want written to the output file.
The material you see here might seem esoteric, but it
is really rather boring.... It is an attempt at "the
right stuff" for a Pachube feed. Note that if you are
attempting the same thing, and your first "datastream"
in your first "feed" has been given ID "0" by you... no
reason not to... then the code below is fine as it is
for automatic update of your datastream.

What are we doing below? Most strings going to the
memo are hard coded, but don't overlook the place where
whatever was passed to FillOutputMemo in sTmp is made
a part of the memo*)
begin
meOutput.lines.clear;
meOutput.lines.add('<?xml version="1.0" encoding="UTF-8"?>');
meOutput.lines.add('<eeml xmlns="http://www.eeml.org/xsd/005">');
meOutput.lines.add('  <environment>');
meOutput.lines.add('    <data id="0">');
sTmp2:='      <value>'+sTmp+'</value>';
meOutput.lines.add(sTmp2);
meOutput.lines.add('    </data>');
meOutput.lines.add('  </environment>');
meOutput.lines.add('</eeml>');
end;


procedure TDD086f1.WriteMemoToFile;
//Should be made a boolean function, returning a
//   code to report success of failure of Write attempt.
begin
//Overwrites previously existing output file without asking.
//Works even if there is no previously existing file.
sTmp:=sBuildPathPlusFile(eSaveToP.text,eSaveToF.text);
meOutput.Lines.SaveToFile(sTmp);
end;

function TDD086f1.sBuildPathPlusFile(sPath, sName:string):string;
//Pick up two strings, e.g. C:/ and tmp.tst, and concatenate.
//So far, this is just a simple concat, but makes seeing what is
//   going on as program execution progresses easier.
//Could be refined to validate result as a proper path+filename
begin
result:=sPath+sName;
end;

end.
